@@ -7,12 +7,8 @@ from shutil import rmtree
 
 from bs4 import BeautifulSoup
 
-from .utils import CommonEqualityMixin
 
-
-# Two lines are considered equals if both are instances of DwCALine and
-# share the same properties
-class DwCALine(CommonEqualityMixin):
+class DwCALine():
     # TODO: if core line: display the number of related extension lines ?
     # TODO: test string representation
     def __str__(self):
@@ -67,6 +63,8 @@ class DwCALine(CommonEqualityMixin):
 
         self.rowtype = my_meta['rowType']
 
+        # TODO: Move line/field stripping to _EmbeddedCSV ??
+
         # fields is a list of the line's content
         line_ending = my_meta['linesTerminatedBy'].decode("string-escape")
         field_ending = my_meta['fieldsTerminatedBy'].decode("string-escape")
@@ -77,6 +75,9 @@ class DwCALine(CommonEqualityMixin):
 
         # If core, we have an id; if extension a coreid
         # TODO: ensure in the norm this is always true
+        self.id = None
+        self.core_id = None
+
         if self.from_core:
             self.id = fields[int(my_meta.id['index'])]
         else:
@@ -101,7 +102,7 @@ class DwCALine(CommonEqualityMixin):
 
         if self.from_core:
             for ext_meta in metadata.findAll('extension'):
-                csv = _DwCACSVIterator(ext_meta, unzipped_folder)
+                csv = _EmbeddedCSV(ext_meta, unzipped_folder)
                 for l in csv.lines():
                     tmp = DwCALine(l, False, ext_meta)
                     if tmp.core_id == self.id:
@@ -126,6 +127,21 @@ class DwCALine(CommonEqualityMixin):
 
         self.source_metadata = m
 
+    # Equality, hashing, ...
+    def __key(self):
+        """Returns a tuple representing the line. Common ground between equality and hash."""
+        return (self.from_core, self.from_extension, self.id, self.core_id, self.data,
+                self.extensions, self.source_metadata)
+
+    def __eq__(self, other):
+        return self.__key() == other.__key()
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
+    def __hash__(self):
+        return hash(self.__key())
+
 
 class DwCAReader(object):
     # Define __enter__ and __exit__ to be used with the 'with' statement
@@ -150,8 +166,8 @@ class DwCAReader(object):
         self.core_rowtype = self._get_core_type()
         self.extensions_rowtype = self._get_extensions_types()
 
-        self._corefile = _DwCACSVIterator(self._metaxml.core,
-                                          self._unzipped_folder_path)
+        self._corefile = _EmbeddedCSV(self._metaxml.core,
+                                      self._unzipped_folder_path)
 
     @property
     #TODO: decide, test and document what we guarantee about ordering
@@ -283,16 +299,15 @@ class GBIFResultsReader(DwCAReader):
         return self._read_additional_file('rights.txt')
 
 
-class _DwCACSVIterator:
-    """Simple, internal use class used to iterate on a DwcA-enclosed CSV file."""
+class _EmbeddedCSV:
+    """Internal use class used to encapsulate a DwcA-enclosed CSV file and its metadata."""
     # TODO: Test this class
     # Not done yet cause issues there will probably make DwCAReader tests fails anyway
+    # In the future it could make sense to make it public
     def __init__(self, metadata_section, unzipped_folder_path):
-        """Initialize the iterator.
-
-        metadata_section: <core> or <extension> section of metaxml concerning the file to iterate.
-        unzipped_folder_path: absolute path to the directory containing the unzipped archive.
-        """
+        #metadata_section: <core> or <extension> section of metaxml concerning the file to iterate.
+        #unzipped_folder_path: absolute path to the directory containing the unzipped archive.
+        
         self._metadata_section = metadata_section
         self._unzipped_folder_path = unzipped_folder_path
 
@@ -312,6 +327,24 @@ class _DwCACSVIterator:
             else:
                 yield line
 
+    @property
+    def headers(self):
+        """Returns a list of (ordered) column names that can be used to create a header line."""
+        field_tags = self._metadata_section.find_all('field')
+
+        columns = {}
+        
+        for tag in field_tags:
+            columns[int(tag['index'])] = tag['term']
+
+        # In addition to DwC terms, we may also have id or core_id columns
+        if self._metadata_section.id:
+            columns[int(self._metadata_section.id['index'])] = 'id'
+        if self._metadata_section.coreid:
+            columns[int(self._metadata_section.id['coreindex'])] = 'coreid'
+
+        return [columns[f] for f in sorted(columns.iterkeys())]
+
     @property
     def filepath(self):
         """Returns the absolute path to the 'subject' file."""
